@@ -9,4 +9,8 @@ mod outer_mod {
 use self::outer_mod::await::await; //~ ERROR expected identifier
     //~^ ERROR expected identifier, found reserved keyword `await`
 
-fn main() {}
+macro_rules! await { () => {}; } //~ ERROR expected identifier, found reserved keyword `await`
+
+fn main() {
+    await!(); //~ ERROR expected expression, found `)`
+}
