@@ -1,4 +1,4 @@
-//! Support for nightly features in Cargo itself
+//! Support for nightly features in Cargo itself.
 //!
 //! This file is the version of `feature_gate.rs` in upstream Rust for Cargo
 //! itself and is intended to be the avenue for which new features in Cargo are
@@ -21,7 +21,7 @@
 //! 3. To actually perform the feature gate, you'll want to have code that looks
 //!    like:
 //!
-//! ```rust,ignore
+//! ```rust,compile_fail
 //! use core::{Feature, Features};
 //!
 //! let feature = Feature::launch_into_space();
@@ -37,7 +37,7 @@
 //!
 //! 4. Update the unstable documentation at
 //!    `src/doc/src/reference/unstable.md` to include a short description of
-//!    how to use your new feature.  When the feature is stabilized, be sure
+//!    how to use your new feature. When the feature is stabilized, be sure
 //!    that the Cargo Guide or Reference is updated to fully document the
 //!    feature and remove the entry from the Unstable section.
 //!
@@ -51,8 +51,13 @@ use std::fmt;
 use std::str::FromStr;
 
 use failure::Error;
+use serde::{Deserialize, Serialize};
 
-use util::errors::CargoResult;
+use crate::util::errors::CargoResult;
+
+pub const SEE_CHANNELS: &str =
+    "See https://doc.rust-lang.org/book/appendix-07-nightly-rust.html for more information \
+     about Rust release channels.";
 
 /// The edition of the compiler (RFC 2052)
 #[derive(Clone, Copy, Debug, Hash, PartialOrd, Ord, Eq, PartialEq, Serialize, Deserialize)]
@@ -64,7 +69,7 @@ pub enum Edition {
 }
 
 impl fmt::Display for Edition {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match *self {
             Edition::Edition2015 => f.write_str("2015"),
             Edition::Edition2018 => f.write_str("2018"),
@@ -77,14 +82,16 @@ impl FromStr for Edition {
         match s {
             "2015" => Ok(Edition::Edition2015),
             "2018" => Ok(Edition::Edition2018),
-            s => {
-                bail!("supported edition values are `2015` or `2018`, but `{}` \
-                       is unknown", s)
-            }
+            s => failure::bail!(
+                "supported edition values are `2015` or `2018`, but `{}` \
+                 is unknown",
+                s
+            ),
         }
     }
 }
 
+#[derive(PartialEq)]
 enum Status {
     Stable,
     Unstable,
@@ -106,7 +113,7 @@ macro_rules! features {
             $(
                 pub fn $feature() -> &'static Feature {
                     fn get(features: &Features) -> bool {
-                        features.$feature
+                        stab!($stab) == Status::Stable || features.$feature
                     }
                     static FEAT: Feature = Feature {
                         name: stringify!($feature),
@@ -147,17 +154,17 @@ macro_rules! stab {
     };
 }
 
-/// A listing of all features in Cargo
-///
-/// "look here"
-///
-/// This is the macro that lists all stable and unstable features in Cargo.
-/// You'll want to add to this macro whenever you add a feature to Cargo, also
-/// following the directions above.
-///
-/// Note that all feature names here are valid Rust identifiers, but the `_`
-/// character is translated to `-` when specified in the `cargo-features`
-/// manifest entry in `Cargo.toml`.
+// A listing of all features in Cargo.
+//
+// "look here"
+//
+// This is the macro that lists all stable and unstable features in Cargo.
+// You'll want to add to this macro whenever you add a feature to Cargo, also
+// following the directions above.
+//
+// Note that all feature names here are valid Rust identifiers, but the `_`
+// character is translated to `-` when specified in the `cargo-features`
+// manifest entry in `Cargo.toml`.
 features! {
     pub struct Features {
 
@@ -170,15 +177,16 @@ features! {
         [unstable] test_dummy_unstable: bool,
 
         // Downloading packages from alternative registry indexes.
-        [unstable] alternative_registries: bool,
+        [stable] alternative_registries: bool,
 
         // Using editions
-        [unstable] edition: bool,
+        [stable] edition: bool,
 
         // Renaming a package in the manifest via the `package` key
-        [unstable] rename_dependency: bool,
+        [stable] rename_dependency: bool,
 
         // Whether a lock file is published with this crate
+        // This is deprecated, and will likely be removed in a future version.
         [unstable] publish_lockfile: bool,
 
         // Overriding profiles for dependencies.
@@ -188,7 +196,13 @@ features! {
         [unstable] namespaced_features: bool,
 
         // "default-run" manifest option,
-        [unstable] default_run: bool,
+        [stable] default_run: bool,
+
+        // Declarative build scripts.
+        [unstable] metabuild: bool,
+
+        // Specifying the 'public' attribute on dependencies
+        [unstable] public_dependency: bool,
     }
 }
 
@@ -210,11 +224,11 @@ impl Features {
     fn add(&mut self, feature: &str, warnings: &mut Vec<String>) -> CargoResult<()> {
         let (slot, status) = match self.status(feature) {
             Some(p) => p,
-            None => bail!("unknown cargo feature `{}`", feature),
+            None => failure::bail!("unknown cargo feature `{}`", feature),
         };
 
         if *slot {
-            bail!("the cargo feature `{}` has already been activated", feature);
+            failure::bail!("the cargo feature `{}` has already been activated", feature);
         }
 
         match status {
@@ -227,11 +241,13 @@ impl Features {
                 );
                 warnings.push(warning);
             }
-            Status::Unstable if !nightly_features_allowed() => bail!(
+            Status::Unstable if !nightly_features_allowed() => failure::bail!(
                 "the cargo feature `{}` requires a nightly version of \
-                 Cargo, but this is the `{}` channel",
+                 Cargo, but this is the `{}` channel\n\
+                 {}",
                 feature,
-                channel()
+                channel(),
+                SEE_CHANNELS
             ),
             Status::Unstable => {}
         }
@@ -269,7 +285,7 @@ impl Features {
                 );
                 msg.push_str(&s);
             }
-            bail!("{}", msg);
+            failure::bail!("{}", msg);
         }
     }
 
@@ -307,20 +323,29 @@ impl Features {
 pub struct CliUnstable {
     pub print_im_a_teapot: bool,
     pub unstable_options: bool,
-    pub offline: bool,
     pub no_index_update: bool,
     pub avoid_dev_deps: bool,
     pub minimal_versions: bool,
     pub package_features: bool,
     pub advanced_env: bool,
     pub config_profile: bool,
-    pub compile_progress: bool,
+    pub dual_proc_macros: bool,
+    pub mtime_on_use: bool,
+    pub install_upgrade: bool,
+    pub cache_messages: bool,
+    pub binary_dep_depinfo: bool,
 }
 
 impl CliUnstable {
     pub fn parse(&mut self, flags: &[String]) -> CargoResult<()> {
         if !flags.is_empty() && !nightly_features_allowed() {
-            bail!("the `-Z` flag is only accepted on the nightly channel of Cargo")
+            failure::bail!(
+                "the `-Z` flag is only accepted on the nightly channel of Cargo, \
+                 but this is the `{}` channel\n\
+                 {}",
+                channel(),
+                SEE_CHANNELS
+            );
         }
         for flag in flags {
             self.add(flag)?;
@@ -337,29 +362,66 @@ impl CliUnstable {
             match value {
                 None | Some("yes") => Ok(true),
                 Some("no") => Ok(false),
-                Some(s) => bail!("expected `no` or `yes`, found: {}", s),
+                Some(s) => failure::bail!("expected `no` or `yes`, found: {}", s),
             }
         }
 
         match k {
             "print-im-a-teapot" => self.print_im_a_teapot = parse_bool(v)?,
             "unstable-options" => self.unstable_options = true,
-            "offline" => self.offline = true,
             "no-index-update" => self.no_index_update = true,
             "avoid-dev-deps" => self.avoid_dev_deps = true,
             "minimal-versions" => self.minimal_versions = true,
             "package-features" => self.package_features = true,
             "advanced-env" => self.advanced_env = true,
             "config-profile" => self.config_profile = true,
-            "compile-progress" => self.compile_progress = true,
-            _ => bail!("unknown `-Z` flag specified: {}", k),
+            "dual-proc-macros" => self.dual_proc_macros = true,
+            "mtime-on-use" => self.mtime_on_use = true,
+            "install-upgrade" => self.install_upgrade = true,
+            "cache-messages" => self.cache_messages = true,
+            "binary-dep-depinfo" => self.binary_dep_depinfo = true,
+            _ => failure::bail!("unknown `-Z` flag specified: {}", k),
         }
 
         Ok(())
     }
+
+    /// Generates an error if `-Z unstable-options` was not used.
+    /// Intended to be used when a user passes a command-line flag that
+    /// requires `-Z unstable-options`.
+    pub fn fail_if_stable_opt(&self, flag: &str, issue: u32) -> CargoResult<()> {
+        if !self.unstable_options {
+            let see = format!(
+                "See https://github.com/rust-lang/cargo/issues/{} for more \
+                 information about the `{}` flag.",
+                issue, flag
+            );
+            if nightly_features_allowed() {
+                failure::bail!(
+                    "the `{}` flag is unstable, pass `-Z unstable-options` to enable it\n\
+                     {}",
+                    flag,
+                    see
+                );
+            } else {
+                failure::bail!(
+                    "the `{}` flag is unstable, and only available on the nightly channel \
+                     of Cargo, but this is the `{}` channel\n\
+                     {}\n\
+                     {}",
+                    flag,
+                    channel(),
+                    SEE_CHANNELS,
+                    see
+                );
+            }
+        }
+        Ok(())
+    }
 }
 
-fn channel() -> String {
+/// Returns the current release channel ("stable", "beta", "nightly", "dev").
+pub fn channel() -> String {
     if let Ok(override_channel) = env::var("__CARGO_TEST_CHANNEL_OVERRIDE_DO_NOT_USE_THIS") {
         return override_channel;
     }
@@ -368,7 +430,7 @@ fn channel() -> String {
             return "dev".to_string();
         }
     }
-    ::version()
+    crate::version()
         .cfg_info
         .map(|c| c.release_channel)
         .unwrap_or_else(|| String::from("dev"))
@@ -394,9 +456,9 @@ thread_local!(
 ///       that called `masquerade_as_nightly_cargo`
 pub fn nightly_features_allowed() -> bool {
     if ENABLE_NIGHTLY_FEATURES.with(|c| c.get()) {
-        return true
+        return true;
     }
-     match &channel()[..] {
+    match &channel()[..] {
         "nightly" | "dev" => NIGHTLY_FEATURES_ALLOWED.with(|c| c.get()),
         _ => false,
     }
