@@ -1,26 +1,35 @@
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 
+use log::{debug, trace};
 use semver::VersionReq;
 use url::Url;
 
-use core::{Dependency, PackageId, Source, SourceId, SourceMap, Summary};
-use core::PackageSet;
-use util::{profile, Config};
-use util::errors::{CargoResult, CargoResultExt};
-use sources::config::SourceConfigMap;
+use crate::core::PackageSet;
+use crate::core::{Dependency, PackageId, Source, SourceId, SourceMap, Summary};
+use crate::sources::config::SourceConfigMap;
+use crate::util::errors::{CargoResult, CargoResultExt};
+use crate::util::{profile, Config};
 
 /// Source of information about a group of packages.
 ///
 /// See also `core::Source`.
 pub trait Registry {
     /// Attempt to find the packages that match a dependency request.
-    fn query(&mut self, dep: &Dependency, f: &mut FnMut(Summary)) -> CargoResult<()>;
+    fn query(
+        &mut self,
+        dep: &Dependency,
+        f: &mut dyn FnMut(Summary),
+        fuzzy: bool,
+    ) -> CargoResult<()>;
 
-    fn query_vec(&mut self, dep: &Dependency) -> CargoResult<Vec<Summary>> {
+    fn query_vec(&mut self, dep: &Dependency, fuzzy: bool) -> CargoResult<Vec<Summary>> {
         let mut ret = Vec::new();
-        self.query(dep, &mut |s| ret.push(s))?;
+        self.query(dep, &mut |s| ret.push(s), fuzzy)?;
         Ok(ret)
     }
+
+    fn describe_source(&self, source: SourceId) -> String;
+    fn is_replaced(&self, source: SourceId) -> bool;
 }
 
 /// This structure represents a registry of known packages. It internally
@@ -29,7 +38,7 @@ pub trait Registry {
 ///
 /// The resolution phase of Cargo uses this to drive knowledge about new
 /// packages as well as querying for lists of new packages. It is here that
-/// sources are updated (e.g. network operations) and overrides are
+/// sources are updated (e.g., network operations) and overrides are
 /// handled.
 ///
 /// The general idea behind this registry is that it is centered around the
@@ -37,6 +46,7 @@ pub trait Registry {
 /// a `Source`. Each `Source` in the map has been updated (using network
 /// operations if necessary) and is ready to be queried for packages.
 pub struct PackageRegistry<'cfg> {
+    config: &'cfg Config,
     sources: SourceMap<'cfg>,
 
     // A list of sources which are considered "overrides" which take precedent
@@ -61,6 +71,7 @@ pub struct PackageRegistry<'cfg> {
     source_ids: HashMap<SourceId, (SourceId, Kind)>,
 
     locked: LockedMap,
+    yanked_whitelist: HashSet<PackageId>,
     source_config: SourceConfigMap<'cfg>,
 
     patches: HashMap<Url, Vec<Summary>>,
@@ -81,24 +92,26 @@ impl<'cfg> PackageRegistry<'cfg> {
     pub fn new(config: &'cfg Config) -> CargoResult<PackageRegistry<'cfg>> {
         let source_config = SourceConfigMap::new(config)?;
         Ok(PackageRegistry {
+            config,
             sources: SourceMap::new(),
             source_ids: HashMap::new(),
             overrides: Vec::new(),
             source_config,
             locked: HashMap::new(),
+            yanked_whitelist: HashSet::new(),
             patches: HashMap::new(),
             patches_locked: false,
             patches_available: HashMap::new(),
         })
     }
 
-    pub fn get(self, package_ids: &[PackageId]) -> PackageSet<'cfg> {
+    pub fn get(self, package_ids: &[PackageId]) -> CargoResult<PackageSet<'cfg>> {
         trace!("getting packages; sources={}", self.sources.len());
-        PackageSet::new(package_ids, self.sources)
+        PackageSet::new(package_ids, self.sources, self.config)
     }
 
-    fn ensure_loaded(&mut self, namespace: &SourceId, kind: Kind) -> CargoResult<()> {
-        match self.source_ids.get(namespace) {
+    fn ensure_loaded(&mut self, namespace: SourceId, kind: Kind) -> CargoResult<()> {
+        match self.source_ids.get(&namespace) {
             // We've previously loaded this source, and we've already locked it,
             // so we're not allowed to change it even if `namespace` has a
             // slightly different precise version listed.
@@ -133,35 +146,44 @@ impl<'cfg> PackageRegistry<'cfg> {
         Ok(())
     }
 
-    pub fn add_sources(&mut self, ids: &[SourceId]) -> CargoResult<()> {
-        for id in ids.iter() {
+    pub fn add_sources(&mut self, ids: impl IntoIterator<Item = SourceId>) -> CargoResult<()> {
+        for id in ids {
             self.ensure_loaded(id, Kind::Locked)?;
         }
         Ok(())
     }
 
-    pub fn add_preloaded(&mut self, source: Box<Source + 'cfg>) {
+    pub fn add_preloaded(&mut self, source: Box<dyn Source + 'cfg>) {
         self.add_source(source, Kind::Locked);
     }
 
-    fn add_source(&mut self, source: Box<Source + 'cfg>, kind: Kind) {
-        let id = source.source_id().clone();
+    fn add_source(&mut self, source: Box<dyn Source + 'cfg>, kind: Kind) {
+        let id = source.source_id();
         self.sources.insert(source);
-        self.source_ids.insert(id.clone(), (id, kind));
+        self.source_ids.insert(id, (id, kind));
     }
 
-    pub fn add_override(&mut self, source: Box<Source + 'cfg>) {
-        self.overrides.push(source.source_id().clone());
+    pub fn add_override(&mut self, source: Box<dyn Source + 'cfg>) {
+        self.overrides.push(source.source_id());
         self.add_source(source, Kind::Override);
     }
 
+    pub fn add_to_yanked_whitelist(&mut self, iter: impl Iterator<Item = PackageId>) {
+        let pkgs = iter.collect::<Vec<_>>();
+        for (_, source) in self.sources.sources_mut() {
+            source.add_to_yanked_whitelist(&pkgs);
+        }
+        self.yanked_whitelist.extend(pkgs);
+    }
+
     pub fn register_lock(&mut self, id: PackageId, deps: Vec<PackageId>) {
         trace!("register_lock: {}", id);
         for dep in deps.iter() {
             trace!("\t-> {}", dep);
         }
-        let sub_map = self.locked
-            .entry(id.source_id().clone())
+        let sub_map = self
+            .locked
+            .entry(id.source_id())
             .or_insert_with(HashMap::new);
         let sub_vec = sub_map
             .entry(id.name().to_string())
@@ -179,7 +201,7 @@ impl<'cfg> PackageRegistry<'cfg> {
     ///
     /// Here the `deps` will be resolved to a precise version and stored
     /// internally for future calls to `query` below. It's expected that `deps`
-    /// have had `lock_to` call already, if applicable. (e.g. if a lock file was
+    /// have had `lock_to` call already, if applicable. (e.g., if a lock file was
     /// already present).
     ///
     /// Note that the patch list specified here *will not* be available to
@@ -195,23 +217,29 @@ impl<'cfg> PackageRegistry<'cfg> {
         // Remember that each dependency listed in `[patch]` has to resolve to
         // precisely one package, so that's why we're just creating a flat list
         // of summaries which should be the same length as `deps` above.
-        let unlocked_summaries = deps.iter()
+        let unlocked_summaries = deps
+            .iter()
             .map(|dep| {
-                debug!("registring a patch for `{}` with `{}`", url, dep.name());
+                debug!(
+                    "registering a patch for `{}` with `{}`",
+                    url,
+                    dep.package_name()
+                );
 
                 // Go straight to the source for resolving `dep`. Load it as we
                 // normally would and then ask it directly for the list of summaries
                 // corresponding to this `dep`.
                 self.ensure_loaded(dep.source_id(), Kind::Normal)
                     .chain_err(|| {
-                        format_err!(
+                        failure::format_err!(
                             "failed to load source for a dependency \
                              on `{}`",
-                            dep.name()
+                            dep.package_name()
                         )
                     })?;
 
-                let mut summaries = self.sources
+                let mut summaries = self
+                    .sources
                     .get_mut(dep.source_id())
                     .expect("loaded source not present")
                     .query_vec(dep)?
@@ -219,44 +247,40 @@ impl<'cfg> PackageRegistry<'cfg> {
 
                 let summary = match summaries.next() {
                     Some(summary) => summary,
-                    None => bail!(
+                    None => failure::bail!(
                         "patch for `{}` in `{}` did not resolve to any crates. If this is \
                          unexpected, you may wish to consult: \
                          https://github.com/rust-lang/cargo/issues/4678",
-                        dep.name(),
+                        dep.package_name(),
                         url
                     ),
                 };
                 if summaries.next().is_some() {
-                    bail!(
+                    failure::bail!(
                         "patch for `{}` in `{}` resolved to more than one candidate",
-                        dep.name(),
+                        dep.package_name(),
                         url
                     )
                 }
                 if summary.package_id().source_id().url() == url {
-                    bail!(
+                    failure::bail!(
                         "patch for `{}` in `{}` points to the same source, but \
                          patches must point to different sources",
-                        dep.name(),
+                        dep.package_name(),
                         url
                     );
                 }
                 Ok(summary)
             })
             .collect::<CargoResult<Vec<_>>>()
-            .chain_err(|| format_err!("failed to resolve patches for `{}`", url))?;
+            .chain_err(|| failure::format_err!("failed to resolve patches for `{}`", url))?;
 
         // Note that we do not use `lock` here to lock summaries! That step
         // happens later once `lock_patches` is invoked. In the meantime though
         // we want to fill in the `patches_available` map (later used in the
         // `lock` method) and otherwise store the unlocked summaries in
         // `patches` to get locked in a future call to `lock_patches`.
-        let ids = unlocked_summaries
-            .iter()
-            .map(|s| s.package_id())
-            .cloned()
-            .collect();
+        let ids = unlocked_summaries.iter().map(|s| s.package_id()).collect();
         self.patches_available.insert(url.clone(), ids);
         self.patches.insert(url.clone(), unlocked_summaries);
 
@@ -284,14 +308,14 @@ impl<'cfg> PackageRegistry<'cfg> {
         &self.patches
     }
 
-    fn load(&mut self, source_id: &SourceId, kind: Kind) -> CargoResult<()> {
+    fn load(&mut self, source_id: SourceId, kind: Kind) -> CargoResult<()> {
         (|| {
             debug!("loading source {}", source_id);
-            let source = self.source_config.load(source_id)?;
+            let source = self.source_config.load(source_id, &self.yanked_whitelist)?;
             assert_eq!(source.source_id(), source_id);
 
             if kind == Kind::Override {
-                self.overrides.push(source_id.clone());
+                self.overrides.push(source_id);
             }
             self.add_source(source, kind);
 
@@ -299,14 +323,14 @@ impl<'cfg> PackageRegistry<'cfg> {
             let _p = profile::start(format!("updating: {}", source_id));
             self.sources.get_mut(source_id).unwrap().update()
         })()
-            .chain_err(|| format_err!("Unable to update {}", source_id))?;
+        .chain_err(|| failure::format_err!("Unable to update {}", source_id))?;
         Ok(())
     }
 
     fn query_overrides(&mut self, dep: &Dependency) -> CargoResult<Option<Summary>> {
-        for s in self.overrides.iter() {
+        for &s in self.overrides.iter() {
             let src = self.sources.get_mut(s).unwrap();
-            let dep = Dependency::new_override(&*dep.name(), s);
+            let dep = Dependency::new_override(&*dep.package_name(), s);
             let mut results = src.query_vec(&dep)?;
             if !results.is_empty() {
                 return Ok(Some(results.remove(0)));
@@ -316,16 +340,16 @@ impl<'cfg> PackageRegistry<'cfg> {
     }
 
     /// This function is used to transform a summary to another locked summary
-    /// if possible. This is where the concept of a lockfile comes into play.
+    /// if possible. This is where the concept of a lock file comes into play.
     ///
-    /// If a summary points at a package id which was previously locked, then we
-    /// override the summary's id itself, as well as all dependencies, to be
+    /// If a summary points at a package ID which was previously locked, then we
+    /// override the summary's ID itself, as well as all dependencies, to be
     /// rewritten to the locked versions. This will transform the summary's
     /// source to a precise source (listed in the locked version) as well as
     /// transforming all of the dependencies from range requirements on
     /// imprecise sources to exact requirements on precise sources.
     ///
-    /// If a summary does not point at a package id which was previously locked,
+    /// If a summary does not point at a package ID which was previously locked,
     /// or if any dependencies were added and don't have a previously listed
     /// version, we still want to avoid updating as many dependencies as
     /// possible to keep the graph stable. In this case we map all of the
@@ -354,7 +378,7 @@ To change the dependency graph via an override it's recommended to use the
 `[replace]` feature of Cargo instead of the path override feature. This is
 documented online at the url below for more information.
 
-http://doc.crates.io/specifying-dependencies.html#overriding-dependencies
+https://doc.rust-lang.org/cargo/reference/specifying-dependencies.html#overriding-dependencies
 ";
 
         for dep in override_summary.dependencies() {
@@ -363,27 +387,25 @@ http://doc.crates.io/specifying-dependencies.html#overriding-dependencies
                 continue;
             }
             let msg = format!(
-                "\
-                 path override for crate `{}` has altered the original list of\n\
+                "path override for crate `{}` has altered the original list of\n\
                  dependencies; the dependency on `{}` was either added or\n\
                  modified to not match the previously resolved version\n\n\
                  {}",
                 override_summary.package_id().name(),
-                dep.name(),
+                dep.package_name(),
                 boilerplate
             );
             self.source_config.config().shell().warn(&msg)?;
             return Ok(());
         }
 
-        if let Some(id) = real_deps.get(0) {
+        if let Some(dep) = real_deps.get(0) {
             let msg = format!(
-                "\
-                path override for crate `{}` has altered the original list of
-                dependencies; the dependency on `{}` was removed\n\n
-                {}",
+                "path override for crate `{}` has altered the original list of\n\
+                 dependencies; the dependency on `{}` was removed\n\n\
+                 {}",
                 override_summary.package_id().name(),
-                id.name(),
+                dep.package_name(),
                 boilerplate
             );
             self.source_config.config().shell().warn(&msg)?;
@@ -395,7 +417,12 @@ http://doc.crates.io/specifying-dependencies.html#overriding-dependencies
 }
 
 impl<'cfg> Registry for PackageRegistry<'cfg> {
-    fn query(&mut self, dep: &Dependency, f: &mut FnMut(Summary)) -> CargoResult<()> {
+    fn query(
+        &mut self,
+        dep: &Dependency,
+        f: &mut dyn FnMut(Summary),
+        fuzzy: bool,
+    ) -> CargoResult<()> {
         assert!(self.patches_locked);
         let (override_summary, n, to_warn) = {
             // Look for an override and get ready to query the real source.
@@ -439,7 +466,7 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
                          with `{}`, \
                          looking at sources",
                         patches.len(),
-                        dep.name(),
+                        dep.package_name(),
                         dep.source_id(),
                         dep.version_req()
                     );
@@ -448,16 +475,16 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
                 // Ensure the requested source_id is loaded
                 self.ensure_loaded(dep.source_id(), Kind::Normal)
                     .chain_err(|| {
-                        format_err!(
+                        failure::format_err!(
                             "failed to load source for a dependency \
                              on `{}`",
-                            dep.name()
+                            dep.package_name()
                         )
                     })?;
 
                 let source = self.sources.get_mut(dep.source_id());
                 match (override_summary, source) {
-                    (Some(_), None) => bail!("override found but no real ones"),
+                    (Some(_), None) => failure::bail!("override found but no real ones"),
                     (None, None) => return Ok(()),
 
                     // If we don't have an override then we just ship
@@ -476,7 +503,7 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
                         // already selected, then we skip this `summary`.
                         let locked = &self.locked;
                         let all_patches = &self.patches_available;
-                        return source.query(dep, &mut |summary| {
+                        let callback = &mut |summary: Summary| {
                             for patch in patches.iter() {
                                 let patch = patch.package_id().version();
                                 if summary.package_id().version() == patch {
@@ -484,7 +511,12 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
                                 }
                             }
                             f(lock(locked, all_patches, summary))
-                        });
+                        };
+                        return if fuzzy {
+                            source.fuzzy_query(dep, callback)
+                        } else {
+                            source.query(dep, callback)
+                        };
                     }
 
                     // If we have an override summary then we query the source
@@ -492,14 +524,21 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
                     // the summaries it gives us though.
                     (Some(override_summary), Some(source)) => {
                         if !patches.is_empty() {
-                            bail!("found patches and a path override")
+                            failure::bail!("found patches and a path override")
                         }
                         let mut n = 0;
                         let mut to_warn = None;
-                        source.query(dep, &mut |summary| {
-                            n += 1;
-                            to_warn = Some(summary);
-                        })?;
+                        {
+                            let callback = &mut |summary| {
+                                n += 1;
+                                to_warn = Some(summary);
+                            };
+                            if fuzzy {
+                                source.fuzzy_query(dep, callback)?;
+                            } else {
+                                source.query(dep, callback)?;
+                            }
+                        }
                         (override_summary, n, to_warn)
                     }
                 }
@@ -507,37 +546,56 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
         };
 
         if n > 1 {
-            bail!("found an override with a non-locked list");
+            failure::bail!("found an override with a non-locked list");
         } else if let Some(summary) = to_warn {
             self.warn_bad_override(&override_summary, &summary)?;
         }
         f(self.lock(override_summary));
         Ok(())
     }
+
+    fn describe_source(&self, id: SourceId) -> String {
+        match self.sources.get(id) {
+            Some(src) => src.describe(),
+            None => id.to_string(),
+        }
+    }
+
+    fn is_replaced(&self, id: SourceId) -> bool {
+        match self.sources.get(id) {
+            Some(src) => src.is_replaced(),
+            None => false,
+        }
+    }
 }
 
 fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Summary) -> Summary {
     let pair = locked
-        .get(summary.source_id())
+        .get(&summary.source_id())
         .and_then(|map| map.get(&*summary.name()))
-        .and_then(|vec| vec.iter().find(|&&(ref id, _)| id == summary.package_id()));
+        .and_then(|vec| vec.iter().find(|&&(id, _)| id == summary.package_id()));
 
     trace!("locking summary of {}", summary.package_id());
 
-    // Lock the summary's id if possible
+    // Lock the summary's ID if possible
     let summary = match pair {
         Some(&(ref precise, _)) => summary.override_id(precise.clone()),
         None => summary,
     };
     summary.map_dependencies(|dep| {
-        trace!("\t{}/{}/{}", dep.name(), dep.version_req(), dep.source_id());
+        trace!(
+            "\t{}/{}/{}",
+            dep.package_name(),
+            dep.version_req(),
+            dep.source_id()
+        );
 
         // If we've got a known set of overrides for this summary, then
         // one of a few cases can arise:
         //
         // 1. We have a lock entry for this dependency from the same
         //    source as it's listed as coming from. In this case we make
-        //    sure to lock to precisely the given package id.
+        //    sure to lock to precisely the given package ID.
         //
         // 2. We have a lock entry for this dependency, but it's from a
         //    different source than what's listed, or the version
@@ -552,10 +610,10 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
         // Cases 1/2 are handled by `matches_id` and case 3 is handled by
         // falling through to the logic below.
         if let Some(&(_, ref locked_deps)) = pair {
-            let locked = locked_deps.iter().find(|id| dep.matches_id(id));
-            if let Some(locked) = locked {
+            let locked = locked_deps.iter().find(|&&id| dep.matches_id(id));
+            if let Some(&locked) = locked {
                 trace!("\tfirst hit on {}", locked);
-                let mut dep = dep.clone();
+                let mut dep = dep;
                 dep.lock_to(locked);
                 return dep;
             }
@@ -565,12 +623,12 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
         // all known locked packages to see if they match this dependency.
         // If anything does then we lock it to that and move on.
         let v = locked
-            .get(dep.source_id())
-            .and_then(|map| map.get(&*dep.name()))
-            .and_then(|vec| vec.iter().find(|&&(ref id, _)| dep.matches_id(id)));
-        if let Some(&(ref id, _)) = v {
+            .get(&dep.source_id())
+            .and_then(|map| map.get(&*dep.package_name()))
+            .and_then(|vec| vec.iter().find(|&&(id, _)| dep.matches_id(id)));
+        if let Some(&(id, _)) = v {
             trace!("\tsecond hit on {}", id);
-            let mut dep = dep.clone();
+            let mut dep = dep;
             dep.lock_to(id);
             return dep;
         }
@@ -579,16 +637,16 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
         // this dependency.
         let v = patches.get(dep.source_id().url()).map(|vec| {
             let dep2 = dep.clone();
-            let mut iter = vec.iter().filter(move |p| {
-                dep2.name() == p.name() && dep2.version_req().matches(p.version())
-            });
+            let mut iter = vec
+                .iter()
+                .filter(move |&&p| dep2.matches_ignoring_source(p));
             (iter.next(), iter)
         });
         if let Some((Some(patch_id), mut remaining)) = v {
             assert!(remaining.next().is_none());
             let patch_source = patch_id.source_id();
             let patch_locked = locked
-                .get(patch_source)
+                .get(&patch_source)
                 .and_then(|m| m.get(&*patch_id.name()))
                 .map(|list| list.iter().any(|&(ref id, _)| id == patch_id))
                 .unwrap_or(false);
@@ -596,7 +654,7 @@ fn lock(locked: &LockedMap, patches: &HashMap<Url, Vec<PackageId>>, summary: Sum
             if patch_locked {
                 trace!("\tthird hit on {}", patch_id);
                 let req = VersionReq::exact(patch_id.version());
-                let mut dep = dep.clone();
+                let mut dep = dep;
                 dep.set_version_req(req);
                 return dep;
             }
