@@ -1,12 +1,11 @@
-use support::install::exe;
-use support::is_nightly;
-use support::paths::CargoPathExt;
-use support::registry::Package;
-use support::{basic_manifest, execs, project};
-use glob::glob;
-use support::hamcrest::{assert_that, existing_file, is_not};
+use std::fmt::{self, Write};
 
-#[test]
+use crate::support::install::exe;
+use crate::support::paths::CargoPathExt;
+use crate::support::registry::Package;
+use crate::support::{basic_manifest, project};
+
+#[cargo_test]
 fn check_success() {
     let foo = project()
         .file(
@@ -21,17 +20,21 @@ fn check_success() {
             path = "../bar"
         "#,
         )
-        .file("src/main.rs", "extern crate bar; fn main() { ::bar::baz(); }")
+        .file(
+            "src/main.rs",
+            "extern crate bar; fn main() { ::bar::baz(); }",
+        )
         .build();
-    let _bar = project().at("bar")
+    let _bar = project()
+        .at("bar")
         .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(foo.cargo("check"), execs());
+    foo.cargo("check").run();
 }
 
-#[test]
+#[cargo_test]
 fn check_fail() {
     let foo = project()
         .file(
@@ -46,17 +49,24 @@ fn check_fail() {
             path = "../bar"
         "#,
         )
-        .file("src/main.rs", "extern crate bar; fn main() { ::bar::baz(42); }")
+        .file(
+            "src/main.rs",
+            "extern crate bar; fn main() { ::bar::baz(42); }",
+        )
         .build();
-    let _bar = project().at("bar")
+    let _bar = project()
+        .at("bar")
         .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(foo.cargo("check"), execs().with_status(101));
+    foo.cargo("check")
+        .with_status(101)
+        .with_stderr_contains("[..]this function takes 0 parameters but 1 parameter was supplied")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn custom_derive() {
     let foo = project()
         .file(
@@ -91,7 +101,8 @@ fn main() {
 "#,
         )
         .build();
-    let _bar = project().at("bar")
+    let _bar = project()
+        .at("bar")
         .file(
             "Cargo.toml",
             r#"
@@ -118,10 +129,10 @@ pub fn derive(_input: TokenStream) -> TokenStream {
         )
         .build();
 
-    assert_that(foo.cargo("check"), execs());
+    foo.cargo("check").run();
 }
 
-#[test]
+#[cargo_test]
 fn check_build() {
     let foo = project()
         .file(
@@ -136,19 +147,23 @@ fn check_build() {
             path = "../bar"
         "#,
         )
-        .file("src/main.rs", "extern crate bar; fn main() { ::bar::baz(); }")
+        .file(
+            "src/main.rs",
+            "extern crate bar; fn main() { ::bar::baz(); }",
+        )
         .build();
 
-    let _bar = project().at("bar")
+    let _bar = project()
+        .at("bar")
         .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(foo.cargo("check"), execs());
-    assert_that(foo.cargo("build"), execs());
+    foo.cargo("check").run();
+    foo.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn build_check() {
     let foo = project()
         .file(
@@ -163,37 +178,39 @@ fn build_check() {
             path = "../bar"
         "#,
         )
-        .file("src/main.rs", "extern crate bar; fn main() { ::bar::baz(); }")
+        .file(
+            "src/main.rs",
+            "extern crate bar; fn main() { ::bar::baz(); }",
+        )
         .build();
 
-    let _bar = project().at("bar")
+    let _bar = project()
+        .at("bar")
         .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(foo.cargo("build"), execs());
-    assert_that(foo.cargo("check"), execs());
+    foo.cargo("build -v").run();
+    foo.cargo("check -v").run();
 }
 
 // Checks that where a project has both a lib and a bin, the lib is only checked
 // not built.
-#[test]
+#[cargo_test]
 fn issue_3418() {
     let foo = project()
         .file("src/lib.rs", "")
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        foo.cargo("check").arg("-v"),
-        execs()
-            .with_stderr_contains("[..] --emit=dep-info,metadata [..]"),
-    );
+    foo.cargo("check -v")
+        .with_stderr_contains("[..] --emit=[..]metadata [..]")
+        .run();
 }
 
 // Some weirdness that seems to be caused by a crate being built as well as
 // checked, but in this case with a proc macro too.
-#[test]
+#[cargo_test]
 fn issue_3419() {
     let p = project()
         .file(
@@ -250,11 +267,11 @@ fn issue_3419() {
         )
         .publish();
 
-    assert_that(p.cargo("check"), execs());
+    p.cargo("check").run();
 }
 
 // Check on a dylib should have a different metadata hash than build.
-#[test]
+#[cargo_test]
 fn dylib_check_preserves_build_cache() {
     let p = project()
         .file(
@@ -274,27 +291,24 @@ fn dylib_check_preserves_build_cache() {
         .file("src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
 [..]Compiling foo v0.1.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ),
-    );
+        )
+        .run();
 
-    assert_that(p.cargo("check"), execs());
+    p.cargo("check").run();
 
-    assert_that(
-        p.cargo("build"),
-        execs()
-            .with_stderr("[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]"),
-    );
+    p.cargo("build")
+        .with_stderr("[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]")
+        .run();
 }
 
 // test `cargo rustc --profile check`
-#[test]
+#[cargo_test]
 fn rustc_check() {
     let foo = project()
         .file(
@@ -309,24 +323,21 @@ fn rustc_check() {
             path = "../bar"
         "#,
         )
-        .file("src/main.rs", "extern crate bar; fn main() { ::bar::baz(); }")
+        .file(
+            "src/main.rs",
+            "extern crate bar; fn main() { ::bar::baz(); }",
+        )
         .build();
-    let _bar = project().at("bar")
+    let _bar = project()
+        .at("bar")
         .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(
-        foo.cargo("rustc")
-            .arg("--profile")
-            .arg("check")
-            .arg("--")
-            .arg("--emit=metadata"),
-        execs(),
-    );
+    foo.cargo("rustc --profile check -- --emit=metadata").run();
 }
 
-#[test]
+#[cargo_test]
 fn rustc_check_err() {
     let foo = project()
         .file(
@@ -341,24 +352,26 @@ fn rustc_check_err() {
             path = "../bar"
         "#,
         )
-        .file("src/main.rs", "extern crate bar; fn main() { ::bar::qux(); }")
+        .file(
+            "src/main.rs",
+            "extern crate bar; fn main() { ::bar::qux(); }",
+        )
         .build();
-    let _bar = project().at("bar")
+    let _bar = project()
+        .at("bar")
         .file("Cargo.toml", &basic_manifest("bar", "0.1.0"))
         .file("src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(
-        foo.cargo("rustc")
-            .arg("--profile")
-            .arg("check")
-            .arg("--")
-            .arg("--emit=metadata"),
-        execs().with_status(101),
-    );
+    foo.cargo("rustc --profile check -- --emit=metadata")
+        .with_status(101)
+        .with_stderr_contains("[CHECKING] bar [..]")
+        .with_stderr_contains("[CHECKING] foo [..]")
+        .with_stderr_contains("[..]cannot find function `qux` in [..] `bar`")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn check_all() {
     let p = project()
         .file(
@@ -383,17 +396,15 @@ fn check_all() {
         .file("b/src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("check").arg("--all").arg("-v"),
-        execs()
-            .with_stderr_contains("[..] --crate-name foo src/lib.rs [..]")
-            .with_stderr_contains("[..] --crate-name foo src/main.rs [..]")
-            .with_stderr_contains("[..] --crate-name b b/src/lib.rs [..]")
-            .with_stderr_contains("[..] --crate-name b b/src/main.rs [..]"),
-    );
+    p.cargo("check --all -v")
+        .with_stderr_contains("[..] --crate-name foo src/lib.rs [..]")
+        .with_stderr_contains("[..] --crate-name foo src/main.rs [..]")
+        .with_stderr_contains("[..] --crate-name b b/src/lib.rs [..]")
+        .with_stderr_contains("[..] --crate-name b b/src/main.rs [..]")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn check_virtual_all_implied() {
     let p = project()
         .file(
@@ -409,15 +420,27 @@ fn check_virtual_all_implied() {
         .file("baz/src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(
-        p.cargo("check").arg("-v"),
-        execs()
-            .with_stderr_contains("[..] --crate-name bar bar/src/lib.rs [..]")
-            .with_stderr_contains("[..] --crate-name baz baz/src/lib.rs [..]"),
-    );
+    p.cargo("check -v")
+        .with_stderr_contains("[..] --crate-name bar bar/src/lib.rs [..]")
+        .with_stderr_contains("[..] --crate-name baz baz/src/lib.rs [..]")
+        .run();
 }
 
-#[test]
+#[cargo_test]
+fn exclude_warns_on_non_existing_package() {
+    let p = project().file("src/lib.rs", "").build();
+    p.cargo("check --all --exclude bar")
+        .with_stdout("")
+        .with_stderr(
+            r#"[WARNING] excluded package(s) bar not found in workspace `[CWD]`
+[CHECKING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+"#,
+        )
+        .run();
+}
+
+#[cargo_test]
 fn targets_selected_default() {
     let foo = project()
         .file("src/main.rs", "fn main() {}")
@@ -427,18 +450,16 @@ fn targets_selected_default() {
         .file("benches/bench3.rs", "")
         .build();
 
-    assert_that(
-        foo.cargo("check").arg("-v"),
-        execs()
-            .with_stderr_contains("[..] --crate-name foo src/lib.rs [..]")
-            .with_stderr_contains("[..] --crate-name foo src/main.rs [..]")
-            .with_stderr_does_not_contain("[..] --crate-name example1 examples/example1.rs [..]")
-            .with_stderr_does_not_contain("[..] --crate-name test2 tests/test2.rs [..]")
-            .with_stderr_does_not_contain("[..] --crate-name bench3 benches/bench3.rs [..]"),
-    );
+    foo.cargo("check -v")
+        .with_stderr_contains("[..] --crate-name foo src/lib.rs [..]")
+        .with_stderr_contains("[..] --crate-name foo src/main.rs [..]")
+        .with_stderr_does_not_contain("[..] --crate-name example1 examples/example1.rs [..]")
+        .with_stderr_does_not_contain("[..] --crate-name test2 tests/test2.rs [..]")
+        .with_stderr_does_not_contain("[..] --crate-name bench3 benches/bench3.rs [..]")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn targets_selected_all() {
     let foo = project()
         .file("src/main.rs", "fn main() {}")
@@ -448,18 +469,16 @@ fn targets_selected_all() {
         .file("benches/bench3.rs", "")
         .build();
 
-    assert_that(
-        foo.cargo("check").arg("--all-targets").arg("-v"),
-        execs()
-            .with_stderr_contains("[..] --crate-name foo src/lib.rs [..]")
-            .with_stderr_contains("[..] --crate-name foo src/main.rs [..]")
-            .with_stderr_contains("[..] --crate-name example1 examples/example1.rs [..]")
-            .with_stderr_contains("[..] --crate-name test2 tests/test2.rs [..]")
-            .with_stderr_contains("[..] --crate-name bench3 benches/bench3.rs [..]"),
-    );
+    foo.cargo("check --all-targets -v")
+        .with_stderr_contains("[..] --crate-name foo src/lib.rs [..]")
+        .with_stderr_contains("[..] --crate-name foo src/main.rs [..]")
+        .with_stderr_contains("[..] --crate-name example1 examples/example1.rs [..]")
+        .with_stderr_contains("[..] --crate-name test2 tests/test2.rs [..]")
+        .with_stderr_contains("[..] --crate-name bench3 benches/bench3.rs [..]")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn check_unit_test_profile() {
     let foo = project()
         .file(
@@ -476,17 +495,15 @@ fn check_unit_test_profile() {
         )
         .build();
 
-    assert_that(foo.cargo("check"), execs());
-    assert_that(
-        foo.cargo("check").arg("--profile").arg("test"),
-        execs()
-            .with_status(101)
-            .with_stderr_contains("[..]badtext[..]"),
-    );
+    foo.cargo("check").run();
+    foo.cargo("check --profile test")
+        .with_status(101)
+        .with_stderr_contains("[..]badtext[..]")
+        .run();
 }
 
 // Verify what is checked with various command-line filters.
-#[test]
+#[cargo_test]
 fn check_filters() {
     let p = project()
         .file(
@@ -543,66 +560,58 @@ fn check_filters() {
         )
         .build();
 
-    assert_that(
-        p.cargo("check"),
-        execs()
-            .with_stderr_contains("[..]unused_normal_lib[..]")
-            .with_stderr_contains("[..]unused_normal_bin[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_t1[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_ex1[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_b1[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_[..]"),
-    );
+    p.cargo("check")
+        .with_stderr_contains("[..]unused_normal_lib[..]")
+        .with_stderr_contains("[..]unused_normal_bin[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_t1[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_ex1[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_b1[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_[..]")
+        .run();
     p.root().join("target").rm_rf();
-    assert_that(
-        p.cargo("check").arg("--tests").arg("-v"),
-        execs()
-            .with_stderr_contains("[..] --crate-name foo src/lib.rs [..] --test [..]")
-            .with_stderr_contains("[..] --crate-name foo src/lib.rs --crate-type lib [..]")
-            .with_stderr_contains("[..] --crate-name foo src/main.rs [..] --test [..]")
-            .with_stderr_contains("[..]unused_unit_lib[..]")
-            .with_stderr_contains("[..]unused_unit_bin[..]")
-            .with_stderr_contains("[..]unused_normal_lib[..]")
-            .with_stderr_contains("[..]unused_normal_bin[..]")
-            .with_stderr_contains("[..]unused_unit_t1[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_ex1[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_ex1[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_b1[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_b1[..]")
-            .with_stderr_does_not_contain("[..]--crate-type bin[..]"),
-    );
+    p.cargo("check --tests -v")
+        .with_stderr_contains("[..] --crate-name foo src/lib.rs [..] --test [..]")
+        .with_stderr_contains("[..] --crate-name foo src/lib.rs [..] --crate-type lib [..]")
+        .with_stderr_contains("[..] --crate-name foo src/main.rs [..] --test [..]")
+        .with_stderr_contains("[..]unused_unit_lib[..]")
+        .with_stderr_contains("[..]unused_unit_bin[..]")
+        .with_stderr_contains("[..]unused_normal_lib[..]")
+        .with_stderr_contains("[..]unused_normal_bin[..]")
+        .with_stderr_contains("[..]unused_unit_t1[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_ex1[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_ex1[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_b1[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_b1[..]")
+        .with_stderr_does_not_contain("[..]--crate-type bin[..]")
+        .run();
     p.root().join("target").rm_rf();
-    assert_that(
-        p.cargo("check").arg("--test").arg("t1").arg("-v"),
-        execs()
-            .with_stderr_contains("[..]unused_normal_lib[..]")
-            .with_stderr_contains("[..]unused_unit_t1[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_lib[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_bin[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_bin[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_ex1[..]")
-            .with_stderr_does_not_contain("[..]unused_normal_b1[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_ex1[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_b1[..]"),
-    );
+    p.cargo("check --test t1 -v")
+        .with_stderr_contains("[..]unused_normal_lib[..]")
+        .with_stderr_contains("[..]unused_unit_t1[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_lib[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_bin[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_bin[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_ex1[..]")
+        .with_stderr_does_not_contain("[..]unused_normal_b1[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_ex1[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_b1[..]")
+        .run();
     p.root().join("target").rm_rf();
-    assert_that(
-        p.cargo("check").arg("--all-targets").arg("-v"),
-        execs()
-            .with_stderr_contains("[..]unused_normal_lib[..]")
-            .with_stderr_contains("[..]unused_normal_bin[..]")
-            .with_stderr_contains("[..]unused_normal_t1[..]")
-            .with_stderr_contains("[..]unused_normal_ex1[..]")
-            .with_stderr_contains("[..]unused_normal_b1[..]")
-            .with_stderr_contains("[..]unused_unit_b1[..]")
-            .with_stderr_contains("[..]unused_unit_t1[..]")
-            .with_stderr_contains("[..]unused_unit_lib[..]")
-            .with_stderr_contains("[..]unused_unit_bin[..]")
-            .with_stderr_does_not_contain("[..]unused_unit_ex1[..]"),
-    );
+    p.cargo("check --all-targets -v")
+        .with_stderr_contains("[..]unused_normal_lib[..]")
+        .with_stderr_contains("[..]unused_normal_bin[..]")
+        .with_stderr_contains("[..]unused_normal_t1[..]")
+        .with_stderr_contains("[..]unused_normal_ex1[..]")
+        .with_stderr_contains("[..]unused_normal_b1[..]")
+        .with_stderr_contains("[..]unused_unit_b1[..]")
+        .with_stderr_contains("[..]unused_unit_t1[..]")
+        .with_stderr_contains("[..]unused_unit_lib[..]")
+        .with_stderr_contains("[..]unused_unit_bin[..]")
+        .with_stderr_does_not_contain("[..]unused_unit_ex1[..]")
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn check_artifacts() {
     // Verify which artifacts are created when running check (#4059).
     let p = project()
@@ -612,117 +621,76 @@ fn check_artifacts() {
         .file("examples/ex1.rs", "fn main() {}")
         .file("benches/b1.rs", "")
         .build();
-    assert_that(p.cargo("check"), execs());
-    assert_that(&p.root().join("target/debug/libfoo.rmeta"), existing_file());
-    assert_that(
-        &p.root().join("target/debug/libfoo.rlib"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug").join(exe("foo")),
-        is_not(existing_file()),
-    );
+
+    p.cargo("check").run();
+    assert!(!p.root().join("target/debug/libfoo.rmeta").is_file());
+    assert!(!p.root().join("target/debug/libfoo.rlib").is_file());
+    assert!(!p.root().join("target/debug").join(exe("foo")).is_file());
+    assert_eq!(p.glob("target/debug/deps/libfoo-*.rmeta").count(), 2);
 
     p.root().join("target").rm_rf();
-    assert_that(p.cargo("check").arg("--lib"), execs());
-    assert_that(&p.root().join("target/debug/libfoo.rmeta"), existing_file());
-    assert_that(
-        &p.root().join("target/debug/libfoo.rlib"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug").join(exe("foo")),
-        is_not(existing_file()),
-    );
+    p.cargo("check --lib").run();
+    assert!(!p.root().join("target/debug/libfoo.rmeta").is_file());
+    assert!(!p.root().join("target/debug/libfoo.rlib").is_file());
+    assert!(!p.root().join("target/debug").join(exe("foo")).is_file());
+    assert_eq!(p.glob("target/debug/deps/libfoo-*.rmeta").count(), 1);
 
     p.root().join("target").rm_rf();
-    assert_that(
-        p.cargo("check").arg("--bin").arg("foo"),
-        execs(),
-    );
-    if is_nightly() {
-        // The nightly check can be removed once 1.27 is stable.
-        // Bins now generate `rmeta` files.
-        // See: https://github.com/rust-lang/rust/pull/49289
-        assert_that(&p.root().join("target/debug/libfoo.rmeta"), existing_file());
-    }
-    assert_that(
-        &p.root().join("target/debug/libfoo.rlib"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug").join(exe("foo")),
-        is_not(existing_file()),
-    );
+    p.cargo("check --bin foo").run();
+    assert!(!p.root().join("target/debug/libfoo.rmeta").is_file());
+    assert!(!p.root().join("target/debug/libfoo.rlib").is_file());
+    assert!(!p.root().join("target/debug").join(exe("foo")).is_file());
+    assert_eq!(p.glob("target/debug/deps/libfoo-*.rmeta").count(), 2);
 
     p.root().join("target").rm_rf();
-    assert_that(
-        p.cargo("check").arg("--test").arg("t1"),
-        execs(),
-    );
-    assert_that(
-        &p.root().join("target/debug/libfoo.rmeta"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug/libfoo.rlib"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug").join(exe("foo")),
-        is_not(existing_file()),
-    );
-    assert_eq!(
-        glob(&p.root().join("target/debug/t1-*").to_str().unwrap())
-            .unwrap()
-            .count(),
-        0
-    );
+    p.cargo("check --test t1").run();
+    assert!(!p.root().join("target/debug/libfoo.rmeta").is_file());
+    assert!(!p.root().join("target/debug/libfoo.rlib").is_file());
+    assert!(!p.root().join("target/debug").join(exe("foo")).is_file());
+    assert_eq!(p.glob("target/debug/t1-*").count(), 0);
+    assert_eq!(p.glob("target/debug/deps/libfoo-*.rmeta").count(), 1);
+    assert_eq!(p.glob("target/debug/deps/libt1-*.rmeta").count(), 1);
 
     p.root().join("target").rm_rf();
-    assert_that(
-        p.cargo("check").arg("--example").arg("ex1"),
-        execs(),
-    );
-    assert_that(
-        &p.root().join("target/debug/libfoo.rmeta"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug/libfoo.rlib"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug/examples").join(exe("ex1")),
-        is_not(existing_file()),
-    );
+    p.cargo("check --example ex1").run();
+    assert!(!p.root().join("target/debug/libfoo.rmeta").is_file());
+    assert!(!p.root().join("target/debug/libfoo.rlib").is_file());
+    assert!(!p
+        .root()
+        .join("target/debug/examples")
+        .join(exe("ex1"))
+        .is_file());
+    assert_eq!(p.glob("target/debug/deps/libfoo-*.rmeta").count(), 1);
+    assert_eq!(p.glob("target/debug/examples/libex1-*.rmeta").count(), 1);
 
     p.root().join("target").rm_rf();
-    assert_that(
-        p.cargo("check").arg("--bench").arg("b1"),
-        execs(),
-    );
-    assert_that(
-        &p.root().join("target/debug/libfoo.rmeta"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug/libfoo.rlib"),
-        is_not(existing_file()),
-    );
-    assert_that(
-        &p.root().join("target/debug").join(exe("foo")),
-        is_not(existing_file()),
-    );
-    assert_eq!(
-        glob(&p.root().join("target/debug/b1-*").to_str().unwrap())
-            .unwrap()
-            .count(),
-        0
-    );
+    p.cargo("check --bench b1").run();
+    assert!(!p.root().join("target/debug/libfoo.rmeta").is_file());
+    assert!(!p.root().join("target/debug/libfoo.rlib").is_file());
+    assert!(!p.root().join("target/debug").join(exe("foo")).is_file());
+    assert_eq!(p.glob("target/debug/b1-*").count(), 0);
+    assert_eq!(p.glob("target/debug/deps/libfoo-*.rmeta").count(), 1);
+    assert_eq!(p.glob("target/debug/deps/libb1-*.rmeta").count(), 1);
 }
 
-#[test]
+#[cargo_test]
+fn short_message_format() {
+    let foo = project()
+        .file("src/lib.rs", "fn foo() { let _x: bool = 'a'; }")
+        .build();
+    foo.cargo("check --message-format=short")
+        .with_status(101)
+        .with_stderr_contains(
+            "\
+src/lib.rs:1:27: error[E0308]: mismatched types
+error: aborting due to previous error
+error: Could not compile `foo`.
+",
+        )
+        .run();
+}
+
+#[cargo_test]
 fn proc_macro() {
     let p = project()
         .file(
@@ -762,8 +730,32 @@ fn proc_macro() {
             "#,
         )
         .build();
-    assert_that(
-        p.cargo("check").arg("-v").env("RUST_LOG", "cargo=trace"),
-        execs(),
-    );
+    p.cargo("check -v").env("CARGO_LOG", "cargo=trace").run();
+}
+
+#[cargo_test]
+fn does_not_use_empty_rustc_wrapper() {
+    let p = project().file("src/lib.rs", "").build();
+    p.cargo("check").env("RUSTC_WRAPPER", "").run();
+}
+
+#[cargo_test]
+fn error_from_deep_recursion() -> Result<(), fmt::Error> {
+    let mut big_macro = String::new();
+    writeln!(big_macro, "macro_rules! m {{")?;
+    for i in 0..130 {
+        writeln!(big_macro, "({}) => {{ m!({}); }};", i, i + 1)?;
+    }
+    writeln!(big_macro, "}}")?;
+    writeln!(big_macro, "m!(0);")?;
+
+    let p = project().file("src/lib.rs", &big_macro).build();
+    p.cargo("check --message-format=json")
+        .with_status(101)
+        .with_stdout_contains(
+            "[..]\"message\":\"recursion limit reached while expanding the macro `m`\"[..]",
+        )
+        .run();
+
+    Ok(())
 }
