@@ -2,13 +2,19 @@ use std::borrow::Borrow;
 use std::collections;
 use std::fs;
 
-use crate::support::{lines_match, paths, project};
+use crate::support::{paths, project};
 use cargo::core::{enable_nightly_features, Shell};
 use cargo::util::config::{self, Config};
 use cargo::util::toml::{self, VecStringOrBool as VSOB};
 use serde::Deserialize;
 
-#[test]
+fn lines_match(a: &str, b: &str) -> bool {
+    // Perform a small amount of normalization for filesystem paths before we
+    // send this to the `lines_match` function.
+    crate::support::lines_match(&a.replace("\\", "/"), &b.replace("\\", "/"))
+}
+
+#[cargo_test]
 fn read_env_vars_for_config() {
     let p = project()
         .file(
@@ -84,7 +90,7 @@ fn assert_error<E: Borrow<failure::Error>>(error: E, msgs: &str) {
     }
 }
 
-#[test]
+#[cargo_test]
 fn get_config() {
     write_config(
         "\
@@ -106,7 +112,7 @@ f1 = 123
     assert_eq!(s, S { f1: Some(456) });
 }
 
-#[test]
+#[cargo_test]
 fn config_unused_fields() {
     write_config(
         "\
@@ -144,7 +150,7 @@ warning: unused key `S.unused` in config file `[..]/.cargo/config`
     }
 }
 
-#[test]
+#[cargo_test]
 fn config_load_toml_profile() {
     write_config(
         "\
@@ -238,7 +244,7 @@ codegen-units = 9
     );
 }
 
-#[test]
+#[cargo_test]
 fn config_deserialize_any() {
     // Some tests to exercise deserialize_any for deserializers that need to
     // be told the format.
@@ -283,7 +289,7 @@ c = ['c']
     }
 }
 
-#[test]
+#[cargo_test]
 fn config_toml_errors() {
     write_config(
         "\
@@ -311,7 +317,7 @@ opt-level = 'foo'
     );
 }
 
-#[test]
+#[cargo_test]
 fn load_nested() {
     write_config(
         "\
@@ -347,7 +353,7 @@ asdf = 3
     assert_eq!(n, expected);
 }
 
-#[test]
+#[cargo_test]
 fn get_errors() {
     write_config(
         "\
@@ -402,7 +408,7 @@ big = 123456789
     );
 }
 
-#[test]
+#[cargo_test]
 fn config_get_option() {
     write_config(
         "\
@@ -420,7 +426,7 @@ f1 = 1
     assert_eq!(config.get::<Option<i32>>("bar.zzzz").unwrap(), None);
 }
 
-#[test]
+#[cargo_test]
 fn config_bad_toml() {
     write_config("asdf");
     let config = new_config(&[]);
@@ -437,7 +443,7 @@ Caused by:
     );
 }
 
-#[test]
+#[cargo_test]
 fn config_get_list() {
     write_config(
         "\
@@ -562,7 +568,7 @@ expected a list, but found a integer for `l3` in [..]/.cargo/config",
     );
 }
 
-#[test]
+#[cargo_test]
 fn config_get_other_types() {
     write_config(
         "\
@@ -584,7 +590,7 @@ ns2 = 456
     );
 }
 
-#[test]
+#[cargo_test]
 fn config_relative_path() {
     write_config(&format!(
         "\
@@ -635,7 +641,7 @@ abs = '{}'
     );
 }
 
-#[test]
+#[cargo_test]
 fn config_get_integers() {
     write_config(
         "\
