@@ -1,8 +1,8 @@
-use support::registry::Package;
-use support::{project, Project};
+use crate::support::registry::Package;
+use crate::support::{project, Project};
 
-static WARNING1: &'static str = "Hello! I'm a warning. :)";
-static WARNING2: &'static str = "And one more!";
+static WARNING1: &str = "Hello! I'm a warning. :)";
+static WARNING2: &str = "And one more!";
 
 fn make_lib(lib_src: &str) {
     Package::new("bar", "0.0.1")
@@ -15,7 +15,8 @@ fn make_lib(lib_src: &str) {
             version = "0.0.1"
             build = "build.rs"
         "#,
-        ).file(
+        )
+        .file(
             "build.rs",
             &format!(
                 r#"
@@ -29,7 +30,8 @@ fn make_lib(lib_src: &str) {
         "#,
                 WARNING1, WARNING2
             ),
-        ).file("src/lib.rs", &format!("fn f() {{ {} }}", lib_src))
+        )
+        .file("src/lib.rs", &format!("fn f() {{ {} }}", lib_src))
         .publish();
 }
 
@@ -46,11 +48,12 @@ fn make_upstream(main_src: &str) -> Project {
             [dependencies]
             bar = "*"
         "#,
-        ).file("src/main.rs", &format!("fn main() {{ {} }}", main_src))
+        )
+        .file("src/main.rs", &format!("fn main() {{ {} }}", main_src))
         .build()
 }
 
-#[test]
+#[cargo_test]
 fn no_warning_on_success() {
     make_lib("");
     let upstream = make_upstream("");
@@ -59,15 +62,17 @@ fn no_warning_on_success() {
         .with_stderr(
             "\
 [UPDATING] `[..]` index
-[DOWNLOADING] bar v0.0.1 ([..])
+[DOWNLOADING] crates ...
+[DOWNLOADED] bar v0.0.1 ([..])
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ).run();
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn no_warning_on_bin_failure() {
     make_lib("");
     let upstream = make_upstream("hi()");
@@ -79,13 +84,13 @@ fn no_warning_on_bin_failure() {
         .with_stderr_does_not_contain(&format!("[WARNING] {}", WARNING1))
         .with_stderr_does_not_contain(&format!("[WARNING] {}", WARNING2))
         .with_stderr_contains("[UPDATING] `[..]` index")
-        .with_stderr_contains("[DOWNLOADING] bar v0.0.1 ([..])")
+        .with_stderr_contains("[DOWNLOADED] bar v0.0.1 ([..])")
         .with_stderr_contains("[COMPILING] bar v0.0.1")
         .with_stderr_contains("[COMPILING] foo v0.0.1 ([..])")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn warning_on_lib_failure() {
     make_lib("err()");
     let upstream = make_upstream("");
@@ -96,7 +101,7 @@ fn warning_on_lib_failure() {
         .with_stderr_does_not_contain("hidden stderr")
         .with_stderr_does_not_contain("[COMPILING] foo v0.0.1 ([..])")
         .with_stderr_contains("[UPDATING] `[..]` index")
-        .with_stderr_contains("[DOWNLOADING] bar v0.0.1 ([..])")
+        .with_stderr_contains("[DOWNLOADED] bar v0.0.1 ([..])")
         .with_stderr_contains("[COMPILING] bar v0.0.1")
         .with_stderr_contains(&format!("[WARNING] {}", WARNING1))
         .with_stderr_contains(&format!("[WARNING] {}", WARNING2))
