@@ -2,7 +2,7 @@ use crate::utils::{match_def_path, paths, snippet, span_lint_and_then, walk_ptrs
 use if_chain::if_chain;
 use rustc::hir::{Expr, ExprKind};
 use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
-use rustc::{declare_tool_lint, lint_array};
+use rustc::{declare_lint_pass, declare_tool_lint};
 use rustc_errors::Applicability;
 
 use std::iter;
@@ -27,17 +27,7 @@ declare_clippy_lint! {
     "calling mem::descriminant on non-enum type"
 }
 
-pub struct MemDiscriminant;
-
-impl LintPass for MemDiscriminant {
-    fn get_lints(&self) -> LintArray {
-        lint_array![MEM_DISCRIMINANT_NON_ENUM]
-    }
-
-    fn name(&self) -> &'static str {
-        "MemDiscriminant"
-    }
-}
+declare_lint_pass!(MemDiscriminant => [MEM_DISCRIMINANT_NON_ENUM]);
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for MemDiscriminant {
     fn check_expr(&mut self, cx: &LateContext<'a, 'tcx>, expr: &'tcx Expr) {
@@ -45,8 +35,8 @@ impl<'a, 'tcx> LateLintPass<'a, 'tcx> for MemDiscriminant {
             if let ExprKind::Call(ref func, ref func_args) = expr.node;
             // is `mem::discriminant`
             if let ExprKind::Path(ref func_qpath) = func.node;
-            if let Some(def_id) = cx.tables.qpath_def(func_qpath, func.hir_id).opt_def_id();
-            if match_def_path(cx.tcx, def_id, &paths::MEM_DISCRIMINANT);
+            if let Some(def_id) = cx.tables.qpath_res(func_qpath, func.hir_id).opt_def_id();
+            if match_def_path(cx, def_id, &paths::MEM_DISCRIMINANT);
             // type is non-enum
             let ty_param = cx.tables.node_substs(func.hir_id).type_at(0);
             if !ty_param.is_enum();
