@@ -1,13 +1,12 @@
 // error-pattern:cargo-clippy
 #![feature(plugin_registrar)]
 #![feature(rustc_private)]
-#![feature(tool_lints)]
-#![allow(unknown_lints)]
-#![allow(clippy::missing_docs_in_private_items)]
 #![warn(rust_2018_idioms)]
 
 // FIXME: switch to something more ergonomic here, once available.
-// (currently there is no way to opt into sysroot crates w/o `extern crate`)
+// (Currently there is no way to opt into sysroot crates without `extern crate`.)
+#[allow(unused_extern_crates)]
+extern crate rustc_driver;
 #[allow(unused_extern_crates)]
 extern crate rustc_plugin;
 use self::rustc_plugin::Registry;
