@@ -1,5 +1,3 @@
-#![feature(tool_lints)]
-
 fn main() {}
 
 #[allow(clippy::result_unwrap_used)]
@@ -7,4 +5,5 @@ fn temporary_cstring() {
     use std::ffi::CString;
 
     CString::new("foo").unwrap().as_ptr();
+    CString::new("foo").expect("dummy").as_ptr();
 }
