@@ -1,17 +1,16 @@
-#![feature(tool_lints)]
-
-
-#[warn(clippy::precedence)]
-#[allow(clippy::identity_op)]
-#[allow(clippy::eq_op)]
+// run-rustfix
+#![warn(clippy::precedence)]
+#![allow(unused_must_use, clippy::no_effect, clippy::unnecessary_operation)]
+#![allow(clippy::identity_op)]
+#![allow(clippy::eq_op)]
 
 macro_rules! trip {
-   ($a:expr) => {
-    match $a & 0b1111_1111i8 {
-        0 => println!("a is zero ({})", $a),
-        _ => println!("a is {}", $a),
-    }
-   };
+    ($a:expr) => {
+        match $a & 0b1111_1111u8 {
+            0 => println!("a is zero ({})", $a),
+            _ => println!("a is {}", $a),
+        }
+    };
 }
 
 fn main() {
